@@ -2,7 +2,7 @@
    rdesktop: A Remote Desktop Protocol client.
    Copyright (C) Matthew Chapman <matthewc.unsw.edu.au> 1999-2008
    Copyright 2004-2011 Peter Astrand <astrand@cendio.se> for Cendio AB
-   Copyright 2010-2011 Henrik Andersson <hean01@cendio.se> for Cendio AB
+   Copyright 2010-2013 Henrik Andersson <hean01@cendio.se> for Cendio AB
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -379,6 +379,14 @@ rdpdr_process_irp(STREAM s)
 	buffer = (uint8 *) xmalloc(1024);
 	buffer[0] = 0;
 
+	if (device >= RDPDR_MAX_DEVICES)
+	{
+		error("invalid irp device 0x%lx file 0x%lx id 0x%lx major 0x%lx minor 0x%lx\n",
+		      device, file, id, major, minor);
+		xfree(buffer);
+		return;
+	}
+
 	switch (g_rdpdr_device[device].device_type)
 	{
 		case DEVICE_TYPE_SERIAL:
@@ -413,6 +421,7 @@ rdpdr_process_irp(STREAM s)
 		default:
 
 			error("IRP for bad device %ld\n", device);
+			xfree(buffer);
 			return;
 	}
 
